﻿using System;

using Xamarin.Forms;

namespace Forum125552Testing
{
	public class PopupControl : ContentView
	{
		
		public enum PopupLocation
		{
			Top,
			Bottom
		}

		private View content;

		private View popup;

		private readonly RelativeLayout layout;

		public PopupControl()
		{
			base.Content = this.layout = new RelativeLayout();
		}

		public new View Content
		{
			get { return this.content; }
			set
			{
				if (this.content != null)
				{
					this.layout.Children.Remove(this.content);
				}

				this.content = value;
				this.layout.Children.Add(this.content, () => this.Bounds);
			}
		}


		public bool IsPopupActive
		{
			get { return this.popup != null; }
		}


		public void ShowPopup(View popupView)
		{
			this.ShowPopup(
				popupView,
				Constraint.RelativeToParent(p => (this.Width - this.popup.WidthRequest) / 2),
				Constraint.RelativeToParent(p => (this.Height - this.popup.HeightRequest) / 2)
				);
		}


		public void ShowPopup(View popupView, Constraint xConstraint, Constraint yConstraint, Constraint widthConstraint = null, Constraint heightConstraint = null)
		{
			DismissPopup();
			this.popup = popupView;

			this.layout.InputTransparent = true;
			if(this.Content != null)
			this.content.InputTransparent = true;
			this.layout.Children.Add(this.popup, xConstraint, yConstraint, widthConstraint, heightConstraint);

			this.layout.ForceLayout();
		}



		public void ShowPopup(View popupView, View presenter, PopupLocation location, float paddingX = 0, float paddingY = 0)
		{
			DismissPopup();
			this.popup = popupView;

			Constraint constraintX = null, constraintY = null;

			switch (location)
			{
				case PopupLocation.Bottom:
					constraintX = Constraint.RelativeToParent(parent => presenter.X + (presenter.Width - this.popup.WidthRequest) / 2);
					constraintY = Constraint.RelativeToParent(parent => parent.Y + presenter.Y + presenter.Height + paddingY);
					break;
				case PopupLocation.Top:
					constraintX = Constraint.RelativeToParent(parent => presenter.X + (presenter.Width - this.popup.WidthRequest) / 2);
					constraintY = Constraint.RelativeToParent(parent =>
						parent.Y + presenter.Y - this.popup.HeightRequest / 2 - paddingY);
					break;
					
			}

			this.ShowPopup(popupView, constraintX, constraintY);
		}


		public void DismissPopup()
		{
			if (this.popup != null)
			{
				this.layout.Children.Remove(this.popup);
				this.popup = null;
			}

			this.layout.InputTransparent = false;

			if (this.content != null)
			{
				this.content.InputTransparent = false;
			}
		}
	}
}


